/*--------------------------------*- C++ -*----------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     | Website:  https://openfoam.org
    \\  /    A nd           | Version:  dev
     \\/     M anipulation  |
\*---------------------------------------------------------------------------*/
FoamFile
{
    version     2.0;
    format      ascii;
    class       dictionary;
    object      blockMeshDict;
}
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

// General macros to create 2D/extruded-2D meshes

changecom(//)changequote([,])
define(calc, [esyscmd(perl -e 'print ($1)')])
define(VCOUNT, 0)
define(vlabel, [[// ]Vertex $1 = VCOUNT define($1, VCOUNT)define([VCOUNT], incr(VCOUNT))])
define(hex2D, hex ($1b $2b $3b $4b $1f $2f $3f $4f)) // Give 2D vertices in anti-clockwise direction
define(patch2D, ($1f $1b $2b $2f)) // Give patch points in anti-clockwise direction, considering contour
define(emptyPatches2D, ($1f $2f $3f $4f) ($1b $4b $3b $2b)) // Give 2D vertices in anti-clockwise direction

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

//        X0     X1     X2    X3        X4   X5    X6      X7           X8
//
//  Y6    18-----20-----22----23--------24---25----27-----29-----------31 Y6
//        |      |      |     |         |    |     |       |            |
//        |      |      |     |         |    |     |       |            |
// Air    |  XI  |  X   |  XI |         |    |     |       |            | Air
// Outlet |      |      |     |   XII   |XIII| XIV |  XV   |    XVI     | Inlet
//        |      |      |     |         |    |     |       |            |
//  Y5    17-----19-----21\_  |         |    |     |       |            | Y5
//                          \_|         |    |     |       |            |
//  Y4           1------3-----5---------7----9_----26-----28-----------30 Y4
//         Water |      |     |         |    | \_
//  Y3     Inlet |   I  |  II |   III   | IV |   \_11-----13              Y3
//  Y2           0------2-----4---------6    |     |       |              Y2
//                                       \   |     |       |
//                                        \  |  V  |  VII  |
//                                         \ |     |       |
//                                          \|     |       | Water
//  Y1                                       8-----10-----12 Outlet       Y1
//                                           |     |       |
//                                           |  VI |  VIII |
//                                           |     |       |
//  Y0                                       14----15-----16              Y0
//                                                Water
//                                                Outlet

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

convertToMeters 1;

// x origin and deltas
define(x0, -0.595 ) // air outlet
define(dx1, 0.5   ) // water inlet
define(dx2, 0.045 ) // left plate
define(dx3, 0.045 ) // tip of left plate
define(dx4, 0.475 ) // tank ramp
define(dx5, 0.04  ) // tip of right plate
define(dx6, 0.02  ) // right plate
define(dx7, 0.06  ) // tank
define(dx8, 0.493 ) // air inlet

// y origin and delta
define(y0, -0.0335 ) // tank bottom
define(dy1, 0.02   ) // tank ramp height
define(dy2, 0.0135 ) // water channel bottom
define(dy3, 0.00675) // tank top
define(dy4, 0.00225) // water channel top
define(dy5, 0.0039 ) // air channel bottom
define(dy6, 0.0771 ) // air channel top

// z origin and delta
define(z0,  0.0   )
define(dz,  0.001 )

// Number of cells
define(dx, 0.005)    // delta for experimental section
define(tank_fx, 1)   // factor for tank
define(gas_fx, 2)  // factor for air in and outlet

define(dy, 0.001)    // delta for water channel
define(gas_fy, 2.6)    // factor for air section

define(nz, 1)        // z direction

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

// calculate block x coordinates starting from left side
define(x1,calc(x0 + dx1))
define(x2,calc(x1 + dx2))
define(x3,calc(x2 + dx3))
define(x4,calc(x3 + dx4))
define(x5,calc(x4 + dx5))
define(x6,calc(x5 + dx6))
define(x7,calc(x6 + dx7))
define(x8,calc(x7 + dx8))

// calculate block y coordinates starting from tank bottom
define(y1,calc(y0 + dy1))
define(y2,calc(y1 + dy2))
define(y3,calc(y2 + dy3))
define(y4,calc(y3 + dy4))
define(y5,calc(y4 + dy5))
define(y6,calc(y5 + dy6))

// calculate second z coordinate
define(z1,calc(z0 + dz))

// calculate cells in x direction for each block
define(nx1,calc(int(dx2/dx)))
define(nx2,calc(int(dx3/dx)))
define(nx3,calc(int(dx4/dx)))
define(nx4,calc(int(dx5/dx)))
define(nx5,calc(int(dx6/(tank_fx*dx))))
define(nx6,nx5)
define(nx7,calc(int(dx7/(tank_fx*dx))))
define(nx8,nx7)

define(nx9,calc(int(dx1/(gas_fx*dx))))
define(nx10,calc(int(dx2/(0.7*gas_fx*dx))))
define(nx11,nx2)
define(nx12,nx3)
define(nx13,nx4)
define(nx14,nx5)
define(nx15,calc(int(dx7/(0.7*gas_fx*dx))))
define(nx16,calc(int(dx8/(gas_fx*dx))))

// calculate cells in y direction for gas and liquid
define(liq_ny,calc(int((dy3+dy4)/dy)))
define(tank_ny,calc(int(liq_ny*dy1/(dy2+dy3))))
define(gas_ny,calc(int(dy6/(gas_fy*dy))))

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

vertices
(
    // water inlet layer and tank
    (x1 y2 z0) vlabel( a0b)
    (x1 y4 z0) vlabel( a1b)
    (x2 y2 z0) vlabel( a2b)
    (x2 y4 z0) vlabel( a3b)
    (x3 y2 z0) vlabel( a4b)
    (x3 y4 z0) vlabel( a5b)
    (x4 y2 z0) vlabel( a6b)
    (x4 y4 z0) vlabel( a7b)
    (x5 y1 z0) vlabel( a8b)
    (x5 y4 z0) vlabel( a9b)
    (x6 y1 z0) vlabel(a10b)
    (x6 y3 z0) vlabel(a11b)
    (x7 y1 z0) vlabel(a12b)
    (x7 y3 z0) vlabel(a13b)
    (x5 y0 z0) vlabel(a14b)
    (x6 y0 z0) vlabel(a15b)
    (x7 y0 z0) vlabel(a16b)

    (x1 y2 z1) vlabel( a0f)
    (x1 y4 z1) vlabel( a1f)
    (x2 y2 z1) vlabel( a2f)
    (x2 y4 z1) vlabel( a3f)
    (x3 y2 z1) vlabel( a4f)
    (x3 y4 z1) vlabel( a5f)
    (x4 y2 z1) vlabel( a6f)
    (x4 y4 z1) vlabel( a7f)
    (x5 y1 z1) vlabel( a8f)
    (x5 y4 z1) vlabel( a9f)
    (x6 y1 z1) vlabel(a10f)
    (x6 y3 z1) vlabel(a11f)
    (x7 y1 z1) vlabel(a12f)
    (x7 y3 z1) vlabel(a13f)
    (x5 y0 z1) vlabel(a14f)
    (x6 y0 z1) vlabel(a15f)
    (x7 y0 z1) vlabel(a16f)

    // air inlet layer
    (x0 y5 z0) vlabel(a17b)
    (x0 y6 z0) vlabel(a18b)
    (x1 y5 z0) vlabel(a19b)
    (x1 y6 z0) vlabel(a20b)
    (x2 y5 z0) vlabel(a21b)
    (x2 y6 z0) vlabel(a22b)
    (x3 y6 z0) vlabel(a23b)
    (x4 y6 z0) vlabel(a24b)
    (x5 y6 z0) vlabel(a25b)
    (x6 y4 z0) vlabel(a26b)
    (x6 y6 z0) vlabel(a27b)
    (x7 y4 z0) vlabel(a28b)
    (x7 y6 z0) vlabel(a29b)
    (x8 y4 z0) vlabel(a30b)
    (x8 y6 z0) vlabel(a31b)

    (x0 y5 z1) vlabel(a17f)
    (x0 y6 z1) vlabel(a18f)
    (x1 y5 z1) vlabel(a19f)
    (x1 y6 z1) vlabel(a20f)
    (x2 y5 z1) vlabel(a21f)
    (x2 y6 z1) vlabel(a22f)
    (x3 y6 z1) vlabel(a23f)
    (x4 y6 z1) vlabel(a24f)
    (x5 y6 z1) vlabel(a25f)
    (x6 y4 z1) vlabel(a26f)
    (x6 y6 z1) vlabel(a27f)
    (x7 y4 z1) vlabel(a28f)
    (x7 y6 z1) vlabel(a29f)
    (x8 y4 z1) vlabel(a30f)
    (x8 y6 z1) vlabel(a31f)
);

blocks
(
    // water section layer
    hex2D( a0,  a2,  a3,  a1)  (nx1 liq_ny nz) simpleGrading (1 1 1) // water channel inlet
    hex2D( a2,  a4,  a5,  a3)  (nx2 liq_ny nz) simpleGrading (1 1 1) // water channel connection
    hex2D( a4,  a6,  a7,  a5)  (nx3 liq_ny nz) simpleGrading (1 1 1) // test section
    hex2D( a6,  a8,  a9,  a7)  (nx4 liq_ny nz) simpleGrading (1 1 1) // slope
    hex2D( a8, a10, a11,  a9)  (nx5 liq_ny nz) simpleGrading (1 1 1) // tank top left
    hex2D(a14, a15, a10,  a8)  (nx6 tank_ny nz)simpleGrading (1 1 1) // tank bottom left
    hex2D(a10, a12, a13, a11)  (nx7 liq_ny nz) simpleGrading (1 1 1) // tank top right
    hex2D(a15, a16, a12, a10)  (nx8 tank_ny nz)simpleGrading (1 1 1) // tank bottom right
    // air section layer
    hex2D(a17, a19, a20, a18) (nx9  gas_ny nz) simpleGrading (1   ((50 58 4) (50 42 0.5)) 1) // air outlet channel
    hex2D(a19, a21, a22, a20) (nx10 gas_ny nz) simpleGrading (0.5 ((50 58 4) (50 42 0.5)) 1) // air channel above water inlet channel
    hex2D(a21,  a5, a23, a22) (nx11 gas_ny nz) simpleGrading (1   ((50 58 4) (50 42 0.5)) 1) // slope
    hex2D( a5,  a7, a24, a23) (nx12 gas_ny nz) simpleGrading (1   ((50 58 4) (50 42 0.5)) 1) // test section
    hex2D( a7,  a9, a25, a24) (nx13 gas_ny nz) simpleGrading (1   ((50 58 4) (50 42 0.5)) 1) // slope
    hex2D( a9, a26, a27, a25) (nx14 gas_ny nz) simpleGrading (1   ((50 58 4) (50 42 0.5)) 1) // air channel above right plate
    hex2D(a26, a28, a29, a27) (nx15 gas_ny nz) simpleGrading (2   ((50 58 4) (50 42 0.5)) 1) // air channel above tank right
    hex2D(a28, a30, a31, a29) (nx16 gas_ny nz) simpleGrading (1   ((50 58 4) (50 42 0.5)) 1) // air inlet channel
);

edges
();

boundary
(
    water_inlet
    {
        type mappedPatch;
        sampleMode nearestCell;
        offsetMode uniform;
        offset (dx2 0 0);
        faces
        (
            patch2D(a1, a0)
        );
    }

    air_inlet
    {
        type mappedPatch;
        sampleMode nearestCell;
        offsetMode uniform;
        offset (-dx8 0 0);
        faces
        (
            patch2D(a30, a31)
        );
    }

    water_outlet
    {
        type patch;
        faces
        (
            // tank bottom
            patch2D(a14, a15)
            patch2D(a15, a16)
            // right side tank
            patch2D(a16, a12)
            patch2D(a12, a13)
        );
    }

    air_outlet
    {
        type patch;
        faces
        (
            patch2D(a18, a17)
        );
    }

    walls
    {
        type wall;
        faces
        (
            // bottom water channel
            patch2D( a0,  a2)
            patch2D( a2,  a4)
            patch2D( a4,  a6)
            patch2D( a6,  a8)
            patch2D( a8, a14)
            // left bottom air/top water channel
            patch2D(a17, a19)
            patch2D(a19, a21)
            patch2D(a21,  a5)
            patch2D( a5,  a3)
            patch2D( a3,  a1)
            // right bottom air/top water channel
            patch2D(a13, a11)
            patch2D(a11,  a9)
            patch2D( a9, a26)
            patch2D(a26, a28)
            patch2D(a28, a30)
            // top air channel
            patch2D(a31, a29)
            patch2D(a29, a27)
            patch2D(a27, a25)
            patch2D(a25, a24)
            patch2D(a24, a23)
            patch2D(a23, a22)
            patch2D(a22, a20)
            patch2D(a20, a18)
        );
    }

    frontAndBack
    {
        type empty;
        faces
        (
            // water section layer
            emptyPatches2D( a0,  a2,  a3,  a1)
            emptyPatches2D( a2,  a4,  a5,  a3)
            emptyPatches2D( a4,  a6,  a7,  a5)
            emptyPatches2D( a6,  a8,  a9,  a7)
            emptyPatches2D( a8, a10, a11,  a9)
            emptyPatches2D(a14, a15, a10,  a8)
            emptyPatches2D(a10, a12, a13, a11)
            emptyPatches2D(a15, a16, a12, a10)
            // air section layer
            emptyPatches2D(a17, a19, a20, a18)
            emptyPatches2D(a19, a21, a22, a20)
            emptyPatches2D(a21,  a5, a23, a22)
            emptyPatches2D( a5,  a7, a24, a23)
            emptyPatches2D( a7,  a9, a25, a24)
            emptyPatches2D( a9, a26, a27, a25)
            emptyPatches2D(a26, a28, a29, a27)
            emptyPatches2D(a28, a30, a31, a29)
        );
    }
);

mergePatchPairs
();

// ************************************************************************* //
